#ifndef CHANNELMAP_SERVICE__CC
#define CHANNELMAP_SERVICE__CC
////////////////////////////////////////////////////////////////////////
/// \file ChannelMap_service.cc
///  
/// \brief A service to manipulate channel map, containing activity
///
/// \author Andrey Sheshukov
////////////////////////////////////////////////////////////////////////

#include "DDTBaseDataProducts/HitList.h"

#include "OnlineCalibration/ChannelMapService.h"

namespace novaddt{
MapSource::MapSource(fhicl::ParameterSet const& p):
    type(p.get<std::string>("type")),
    name(p.get<std::string>("name")),
    count(p.get<int>("count"))
    {
        if(type!="file" && type!="product")
        mf::LogError("MapSource")<<"Wrong source type '"<<type<<"'";
    }
}
    
namespace novaddt{
ChannelMapService::ChannelMapService
    (fhicl::ParameterSet const& p, art::ActivityRegistry& reg):
    fMapSource(p.get<fhicl::ParameterSet>("source")),
    fSegName(p.get<std::string>("shmem.segment")),
    fMapName(p.get<std::string>("shmem.object")),
    fSharedMapContainer(fSegName.data(),fMapName.data())
{
    mf::LogInfo("ChannelMapService")<<"\t config:"
    <<"\n\tmap source:"
    <<"\n\t\ttype:"<<fMapSource.type
    <<"\n\t\tname:"<<fMapSource.name
    <<"\n\t\tcount:"<<fMapSource.count
    <<"\nshmem.segment="<<fSegName<<";shmem.object="<<fMapName;
 
    fSharedMapContainer.getData().SetCounterMax(fMapSource.count);

    //register callbacks: before starting run
    if(fMapSource.type=="file")
        reg.sPreBeginRun.watch(&novaddt::ChannelMapService::preBeginRun, *this);
     
    //register callbacks: after processing an event
    if(fMapSource.type=="product")
        reg.sPostProcessEvent.watch(&novaddt::ChannelMapService::postProcessEvent, *this);
}

void ChannelMapService::preBeginRun(const art::Run& run){
    readMapFromFile(fMapSource.name);
}

void ChannelMapService::postProcessEvent(const art::Event& event)
{
   fillMapFromEvent(event,fMapSource.name);
}

void ChannelMapService::readMapFromFile(const std::string& fname)
{
    mf::LogWarning("ChannelMapService")<<"read map from file \""<<fname<<"\": not implemented yet!";
}
void ChannelMapService::saveMapToFile(const std::string& fname)
{
    mf::LogWarning("ChannelMapService")<<"save map to file \""<<fname<<"\": not implemented yet!";
}

void ChannelMapService::fillMapFromEvent
       (const art::Event& event, const art::InputTag& product)
{
    auto hits=event.getValidHandle<novaddt::HitList>(product);
    mf::LogDebug("ChannelMapService")<<"fill map from \""<<product
        <<"\" with "<<hits->size()<<" hits"
        <<"\n Counter="<<getCounter();
    
    addCounter();
    auto lock = GetLock();
    auto &NoiseMap = GetWMap();
    //fill the hits map
    for(const auto &hit: *hits)
        NoiseMap[hit]+=1;
}


calib::sharableLock ChannelMapService::GetLock(){
    return fSharedMapContainer.getData().getLock();
}

const calib::FDHitMap& ChannelMapService::GetRMap() const{
    return fSharedMapContainer.getData().getRBuffer();
}

calib::FDHitMap& ChannelMapService::GetWMap(){
    return fSharedMapContainer.getData().getWBuffer();
}


DEFINE_ART_SERVICE(ChannelMapService);
}
#endif
