////////////////////////////////////////////////////////////////////////
// Class:       HighEFinder
// Module Type: filter
// File:        HighEFinder_module.cc
//
// Generated at Tue Oct 25 22:21:45 2016 by Andrey Sheshukov using artmod
// from cetpkgsupport v1_10_02.
////////////////////////////////////////////////////////////////////////

#include "art/Framework/Core/EDFilter.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Principal/Event.h"
#include "art/Framework/Principal/Handle.h"
#include "art/Framework/Principal/Run.h"
#include "art/Framework/Principal/SubRun.h"
#include "art/Utilities/InputTag.h"
#include "fhiclcpp/ParameterSet.h"
#include "messagefacility/MessageLogger/MessageLogger.h"

#include "DDTBaseDataProducts/HitList.h"
#include "DDTBaseDataProducts/GroupedHitList.h"
#include "SuperNovaDDT/SliceSelector.h"
#include <memory>
#include <list>

namespace supernova {
  class HighEFinder;
}


class supernova::HighEFinder : public art::EDFilter {
public:
  explicit HighEFinder(fhicl::ParameterSet const & p);
  // The destructor generated by the compiler is fine for classes
  // without bare pointers or other resource use.

  // Plugins should not be copied or assigned.
  HighEFinder(HighEFinder const &) = delete;
  HighEFinder(HighEFinder &&) = delete;
  HighEFinder & operator = (HighEFinder const &) = delete;
  HighEFinder & operator = (HighEFinder &&) = delete;

  // Required functions.
  bool filter(art::Event & e) override;
 
  novaddt::HitList GetHitsAroundTime(const novaddt::HitList&, const novaddt::TDC&);

private:

  // parameters
  std::string fSlicesTag;
  std::string fHitsTag;
  std::string fOutputTag;
  novaddt::SliceSelector fSelector;
  int fDTime_min,fDTime_max;
};


supernova::HighEFinder::HighEFinder(fhicl::ParameterSet const & p):
    fSlicesTag   (p.get< std::string >("slices_tag"  )),
    fHitsTag     (p.get< std::string >("hits_tag"  )),
    fOutputTag   (p.get< std::string >("output_tag"  )),
    fSelector    (p.get<fhicl::ParameterSet>("slice_cuts")),
    fDTime_min   (p.get<int>("TimeRange.min",0)),
    fDTime_max   (p.get<int>("TimeRange.max",1000))
{
    produces<std::vector<novaddt::HitList>>(fOutputTag);
    mf::LogInfo("HighEFinder")<<"config:"
        <<"\n slices_tag: "<<fSlicesTag
        <<"\n hits_tag:   "<<fHitsTag
        <<"\n output_tag: "<<fOutputTag
        <<"\n slice_cuts: "<<fSelector
        <<"\n TimeRange: ["<<fDTime_min<<" : "<<fDTime_max<<"]";
}

//a comparison function for searching in time
bool cmp_tdc(const novaddt::DAQHit& h, const novaddt::TDC& tdc){
    return h.TDC()<tdc;
}

novaddt::HitList supernova::HighEFinder::GetHitsAroundTime(
                const novaddt::HitList& allhits, 
                const novaddt::TDC& slice_time
                )
{
       //find the time range
    // Only need to look at the time region around the end
    auto tmin=slice_time+fDTime_min;
    auto tmax=slice_time+fDTime_max;
    const auto range_beg = std::lower_bound(allhits.begin(),allhits.end(),tmin, cmp_tdc);
    const auto range_end = std::lower_bound(range_beg,      allhits.end(),tmax, cmp_tdc);
    mf::LogDebug("HighEFinder")
        <<"retrieving "<<std::distance(range_beg, range_end)<<" hits "
        <<"in time range ["<<tmin<<" : "<<tmax<<"]";

    return novaddt::HitList(range_beg, range_end);
}

bool supernova::HighEFinder::filter(art::Event & e)
{
    auto timeslices=e.getValidHandle<novaddt::GroupedHitList<novaddt::TDC>>(fSlicesTag);
    auto allhits=e.getValidHandle<novaddt::HitList>(fHitsTag);
    //prepare resulting vector
    auto result=std::make_unique<std::vector<novaddt::HitList>>();

    //find the high-energy slice
    for(unsigned NSlice=0; NSlice<timeslices->n_groups();++NSlice){
        const auto& slice = timeslices->get(NSlice);
        if(fSelector.checkSlice(slice)){
            //found the slice, get its starting time
            auto slice_time=slice[0].TDC().val;
            mf::LogInfo("HighEFinder")<<"Found HE slice starting at t="<<slice_time;
            // get the hits in this time range
            result->push_back(GetHitsAroundTime(*allhits,slice_time));
        }
    }
    
    mf::LogDebug("HighEFinder")<<" selected "<<result->size()<<" slices";
    e.put(std::move(result));
    return true;
    //return (!result->empty());
}

DEFINE_ART_MODULE(supernova::HighEFinder)
