#ifndef CLUSTERINFO__H
#define CLUSTERINFO__H
#include <TMath.h>
class hitInfo{
public:
  uint16_t plane, cell;
};

class clusterInfo{
public:
    uint64_t N() const {return n[0]+n[1];}
    uint64_t Nx() const {return n[0];}
    uint64_t Ny() const {return n[1];}
    uint64_t ADC() const{return adc;}
    double ADC_corr() const{return adc_corr;}

    uint64_t t = 0;
    uint64_t crd[3] = {0,0,0};
    uint64_t n[2] = {0,0};
    uint64_t adc= 0;
    double adc_corr= 0;
    double Noise = 0;
    //uint16_t adc[2] = {0,0};
};

clusterInfo CalcCluster(const novaddt::HitList& slice){
  clusterInfo c;
  for(auto const& hit : slice){
    if(c.N() == 0){
      c.crd[2] = hit.Plane().val;
      c.t = hit.TDC().val;
    }

    const uint16_t view = hit.View().val-1;
    if(c.n[view] == 0) c.crd[view] = hit.Cell().val;

    ++c.n[view];
  }

  for(auto const& hit : slice){
    // View of *this* cell
    const uint16_t view = hit.View().val-1;

    // coordinate of the *other* view, or halfway if 
    const double pos = c.n[!view]?c.crd[!view]:384./2.;

    const double atten_len = 400./3.93; // in *cells*

    c.adc      += hit.ADC().val;
    c.adc_corr += hit.ADC().val
                      *exp((384.-pos )/atten_len)
                      /exp((384./ 2. )/atten_len);
  }
  return c;

}

std::ostream& operator << (std::ostream& s,const clusterInfo& c){
        //s<<"adc="<<(int)c.ADC()<<" n=["<<c.n[0]<<","<<c.n[1]<<"]; pos=["<<c.crd[0]<<","<<c.crd[1]<<","<<c.crd[2]<<","<<c.t<<"]";
        s<<c.ADC()<<" "<<c.n[0]<<" "<<c.n[1]<<" "<<c.crd[0]<<" "<<c.crd[1]<<" "<<c.crd[2]<<" "<<c.t;
        return s;
}
#endif
